#ifndef CDECL_H_
#define CEDCL_H_

#include <stddef.h>
#include <stdint.h>

/* Declaration specifier kinds. */
enum {
	CDECL_SPEC_TYPE = 0,
	CDECL_SPEC_STOR = 256,
	CDECL_SPEC_QUAL = 512,
	CDECL_SPEC_FUNC = 1024,
};

enum {
	CDECL_TYPE_VOID = CDECL_SPEC_TYPE,
	CDECL_TYPE_CHAR,
	CDECL_TYPE_SHORT,
	CDECL_TYPE_INT,
	CDECL_TYPE_LONG,
	CDECL_TYPE_FLOAT,
	CDECL_TYPE_DOUBLE,
	CDECL_TYPE_SIGNED,
	CDECL_TYPE_UNSIGNED,
	CDECL_TYPE_BOOL,
	CDECL_TYPE_COMPLEX,
	CDECL_TYPE_STRUCT,
	CDECL_TYPE_UNION,
	CDECL_TYPE_ENUM,
	CDECL_TYPE_IDENT,
	CDECL_STOR_TYPEDEF = CDECL_SPEC_STOR,
	CDECL_STOR_EXTERN,
	CDECL_STOR_STATIC,
	CDECL_STOR_AUTO,
	CDECL_STOR_REGISTER,
	CDECL_QUAL_RESTRICT = CDECL_SPEC_QUAL,
	CDECL_QUAL_VOLATILE,
	CDECL_QUAL_CONST,
	CDECL_FUNC_INLINE = CDECL_SPEC_FUNC,
};

/* Declarator types. */
enum {
	CDECL_DECL_IDENT,
	CDECL_DECL_POINTER,
	CDECL_DECL_ARRAY,
};

struct cdecl {
	struct cdecl_declspec {
		struct cdecl_declspec *next;
		unsigned type;
		char *ident;
	} *specifiers;

	struct cdecl_declarator {
		struct cdecl_declarator *next;
		unsigned type;
		union {
			char *ident;
			struct cdecl_pointer {
				struct cdecl_declspec   *qualifiers;
				struct cdecl_declarator *declarator;
			} pointer;
			struct cdecl_array {
				char *vla;
				uintmax_t length;
				struct cdecl_declarator *declarator;
			} array;
		} u;
	} *declarators;
};

struct cdecl *cdecl_parse_decl(const char *declstr);
void cdecl_free(struct cdecl *decl);

size_t cdecl_explain(char *buf, size_t n, struct cdecl *decl);

static inline int cdecl_spec_kind(struct cdecl_declspec *spec)
{
	return spec->type & ~0xffu;
}

#endif
